{% load bootstrap3 %}
{% load i18n %}
{% bootstrap_form form %}
<div class="alert alert-warning">
    {% blocktrans trimmed %}
        Note that BitPay will create a "refund request" and the user will get an email with instructions on how to
        retrieve the refund. pretix, however, will mark the ticket as refunded immediately.
    {% endblocktrans %}
</div>
