{% load i18n %}

{% if payment_info %}
    {% if "id" in payment_info %}
        <dl class="dl-horizontal">
            <dt>{% trans "Invoice ID" %}</dt>
            <dd>{{ payment_info.id }}</dd>
            <dt>{% trans "Status" %}</dt>
            <dd>{{ payment_info.status }}</dd>
        </dl>
    {% endif %}
    {% if "message" in payment_info %}
        <dl class="dl-horizontal">
            <dt>{% trans "Error message" %}</dt>
            <dd>{{ payment_info.message }}</dd>
        </dl>
    {% endif %}
{% endif %}
