{% load i18n %}

<p>{% blocktrans trimmed %}
    We're waiting for an answer from the payment provider regarding your payment. Please contact us if this
    takes more than a few hours.
{% endblocktrans %}</p>
